class OneSocialError(Exception):
    """
    Базовый класс для ошибок OneSocial.

    code - код ошибки, str или None.
    message - описание ошибки, str.
    """
    def __init__(self, message, code=None):
        self.code = code
        self.message = message

    def __str__(self):
        if self.code:
            return "{}: {}".format(self.code, self.message)
        else:
            return self.message


class OneSocialOAuthError(OneSocialError):
    """
    Ошибка входа через соцсети.

    code - код ошибки, str или None.
    message - описание ошибки, str.
    """
    pass


class OneSocialAPIError(OneSocialError):
    """
    Ошибка API OneSocial (кроме API входа через соцсети).

    code - код ошибки, str или None.
    message - описание ошибки, str.
    """
    pass



from typing import Optional

import requests

from .errors import OneSocialAPIError


class BaseAPI:
    def __init__(self, *, access_token):
        self.access_token = access_token

    def _raise_if_error(self, resp):
        if resp.status_code < 200 or resp.status_code > 299:
            try:
                resp_json = resp.json()
                error_code = resp_json['error_code']
                error_description = resp_json['error_description']
            except ValueError:
                error_code = None
                error_description = resp.text

            raise OneSocialAPIError(error_description, code=error_code)

    def _get_authorization_headers(self):
        return {
            'Authorization': 'Bearer {}'.format(self.access_token),
        }


class UserProfile:
    """
    Профиль пользователя социальной сети.

    Связка network и uid уникальна для каждого аккаунта во всех соцсетях.

    Обязательные поля:

    network - идентификатор соцсети, str.
        См. список поддерживаемых соцсетей:
        https://onesocial.dev/panel/docs/sociallogin/#networks
    uid - ID аккаунта в соцсети, str.
    username - латинизированный юзернейм, который можно использовать
        на сайте, str.
    human_name - Имя Фамилия, str. Если соцсеть не предоставляет имя и фамилию
        пользователя, это поле повторяет username.

    Необязательные поля:

    email - email, str. Может отсутствовать, если не указан в аккаунте
        пользователя или если пользователя запретил доступ к своему email.
    picture - URL аватара, str. Может отсутствовать, если в профиле
        не указан аватар.
    """
    def __init__(
                self, *,
                network: str = None,
                uid: str = None,
                username: str = None,
                human_name: str = None,
                email: Optional[str] = None,
                picture: Optional[str] = None,
            ):
        self.network = network
        self.uid = uid
        self.username = username
        self.human_name = human_name
        self.email = email
        self.picture = picture


class UsersAPI(BaseAPI):
    def me(self):
        """
        Возвращает профиль аккаунта, под которым авторизован клиент.

        https://onesocial.dev/panel/docs/users-api/#get-me
        """
        resp = requests.get(
            'https://onesocial.dev/api/users/me/',
            headers=self._get_authorization_headers(),
        )
        self._raise_if_error(resp)

        resp_json = resp.json()
        return UserProfile(
            network=resp_json['network'],
            uid=resp_json['uid'],
            username=resp_json['username'],
            human_name=resp_json['human_name'],
            email=resp_json['email'],
            picture=resp_json['picture'],
        )



from typing import Optional
from urllib.parse import urlencode

import requests

from .errors import OneSocialOAuthError


class TokenGrant:
    """
    Объект, содержащий токен доступа, выданный OneSocial, и его параметры.

    access_token - токен доступа, str.
    expires_in - срок действия токена с момента выдачи в секундах, int.
    """

    def __init__(self, *, access_token, token_type, expires_in):
        self.access_token = access_token
        self.token_type = token_type
        self.expires_in = expires_in


class OAuth:
    """
    Класс OAuth позволяет работать с API входа через соцсети OneSocial.

    Подробная документация: https://onesocial.dev/panel/docs/sociallogin/
    """

    CODE = 'code'
    TOKEN = 'token'

    def __init__(self, *, client_id=None, client_secret=None):
        """
        Client ID и Client Secret можно получить на странице настроек входа
        через соцсети: https://onesocial.dev/panel/oauthkeys/index/
        """
        self.client_id = client_id
        self.client_secret = client_secret

    def init(
                self, *,
                network: str = None,
                response_type: str = None,
                redirect_uri: str = None,
                state: Optional[str] = None,
            ) -> str:
        """
        Первый шаг в процессе аутентификации пользователя. Этот метод
        возвращает URL страницы, на которую следует направить пользователя
        для запуска аутентификации.

        network - идентификатор соцсети, str.
            См. список поддерживаемых соцсетей:
            https://onesocial.dev/panel/docs/sociallogin/#networks
        response_type - OAuth.CODE или OAuth.TOKEN.
            Этот параметр определяет режим работы OAuth. Для сайтов следует
            использовать OAuth.CODE. См. описание различий между двумя
            режимами:
            https://onesocial.dev/panel/docs/sociallogin/#modes
        redirect_uri - URI, на который следует перенаправить пользователя после
            авторизации, str. Домен этого URI должен быть указан в списке
            доменов в настройках пары ключей Client ID / Client Secret.
        state - любая произвольная строка, опционально, str. OneSocial
            скопирует ее в redirect_uri.

        Возвращает URI страницы запуска аутентификации, str.
        """
        url = 'https://onesocial.dev/api/sociallogin/init/{}/'.format(network)

        query = {
            'client_id': self.client_id,
            'response_type': response_type,
            'redirect_uri': redirect_uri,
        }

        if state:
            query['state'] = state

        return url + '?' + urlencode(query)

    def token(
                self, *,
                code: str = None,
                redirect_uri: str = None,
            ) -> TokenGrant:
        """
        Запрашивает токена доступа по коду авторизации.

        code - код авторизации, str.
        redirect_uri - Redirect URI, который был передан в метод init при
            запуске аутентификации.

        Возвращает объект TokenGrant.
        """
        resp = requests.post('https://onesocial.dev/api/sociallogin/token/', {
            'grant_type': 'authorization_code',
            'code': code,
            'redirect_uri': redirect_uri,
            'client_secret': self.client_secret,
        })

        if resp.status_code < 200 or resp.status_code > 299:
            try:
                resp_json = resp.json()
                error = resp_json['error']
                error_description = resp_json['error_description']
            except ValueError:
                error = None
                error_description = resp.text

            raise OneSocialOAuthError(error_description, code=error)

        resp_json = resp.json()

        return TokenGrant(
            access_token=resp_json['access_token'],
            token_type=resp_json['token_type'],
            expires_in=resp_json['expires_in'],
        )



import pathlib

from setuptools import find_packages, setup

BASE_DIR = pathlib.Path(__file__).parent

README = (BASE_DIR / "README.md").read_text()

setup(
    name="onesocial",
    version="1.0.1",
    description="Клиент для OneSocial API",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/onesocial-dev/onesocial-python",
    author="Oleg Yamnikov",
    author_email="oyam@oxymeal.ru",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["requests>=2.0.0"],
)
